@@ -33,13 +33,9 @@
 /*	Warning: flushing undeliverable mail frequently will result in
 /*	poor delivery performance of all other mail.
 /* .IP "\fB-i \fIqueue_id\fR"
-/*	Schedule immediate delivery of deferred mail with the
-/*	specified queue ID.
-/*
-/*	This option implements the traditional \fBsendmail -qI\fR
-/*	command, by contacting the \fBflush\fR(8) server.
-/*
-/*	This feature is available with Postfix version 2.4 and later.
+/*	Schedule immediate delivery of mail with the specified queue ID.
+/*	This feature uses the \fBflush\fR(8) server, and is available
+/*	with Postfix 2.4 and later.
 /* .IP \fB-p\fR
 /*	Produce a traditional sendmail-style queue listing.
 /*	This option implements the traditional \fBmailq\fR command,
@@ -192,7 +188,6 @@
 
 #include <mail_proto.h>
 #include <mail_params.h>
-#include <mail_version.h>
 #include <mail_conf.h>
 #include <mail_task.h>
 #include <mail_run.h>
@@ -221,18 +216,6 @@
 
  /*
   * Modes of operation.
-  * 
-  * XXX To support flush by recipient domain, or for destinations that have no
-  * mapping to logfile, the server has to defend against resource exhaustion
-  * attacks. A malicious user could fork off a postqueue client that starts
-  * an expensive requests and then kills the client immediately; this way she
-  * could create a high Postfix load on the system without ever exceeding her
-  * own per-user process limit. To prevent this, either the server needs to
-  * establish frequent proof of client liveliness with challenge/response, or
-  * the client needs to restrict expensive requests to privileged users only.
-  * 
-  * We don't have this problem with queue listings. The showq server detects an
-  * EPIPE error after reporting a few queue entries.
   */
 #define PQ_MODE_DEFAULT		0	/* noop */
 #define PQ_MODE_MAILQ_LIST	1	/* list mail queue */
@@ -430,8 +413,6 @@ static NORETURN usage(void)
     msg_fatal_status(EX_USAGE, "usage: postqueue -f | postqueue -i queueid | postqueue -p | postqueue -s site");
 }
 
-MAIL_VERSION_STAMP_DECLARE;
-
 /* main - the main program */
 
 int     main(int argc, char **argv)
@@ -446,11 +427,6 @@ int     main(int argc, char **argv)
     ARGV   *import_env;
     int     bad_site;
 
-    /*
-     * Fingerprint executables and core dumps.
-     */
-    MAIL_VERSION_STAMP_ALLOCATE;
-
     /*
      * Be consistent with file permissions.
      */
@@ -527,8 +503,6 @@ int     main(int argc, char **argv)
      * Further initialization...
      */
     mail_conf_read();
-    if (strcmp(var_syslog_name, DEF_SYSLOG_NAME) != 0)
-	msg_syslog_init(mail_task("postqueue"), LOG_PID, LOG_FACILITY);
     mail_dict_init();				/* proxy, sql, ldap */
     get_mail_conf_str_table(str_table);
 
