@@ -120,9 +120,6 @@ extern void vstream_control(VSTREAM *, int,...);
 #define VSTREAM_CTL_TIMEOUT	8
 #define VSTREAM_CTL_EXCEPT	9
 #define VSTREAM_CTL_CONTEXT	10
-#ifdef F_DUPFD
-#define VSTREAM_CTL_DUPFD	11
-#endif
 
 extern VSTREAM *PRINTFLIKE(1, 2) vstream_printf(const char *,...);
 extern VSTREAM *PRINTFLIKE(2, 3) vstream_fprintf(VSTREAM *, const char *,...);
@@ -153,12 +150,6 @@ extern ssize_t vstream_peek(VSTREAM *);
 #define vstream_setjmp(stream)		setjmp((stream)->jbuf[0])
 #define vstream_longjmp(stream, val)	longjmp((stream)->jbuf[0], (val))
 
- /*
-  * Tweaks and workarounds.
-  */
-extern int vstream_tweak_sock(VSTREAM *);
-extern int vstream_tweak_tcp(VSTREAM *);
-
 /* LICENSE
 /* .ad
 /* .fi
